/*
 * qfits example: Manipulating FITS headers.
 *
 * The following example shows how to create and manipulate a FITS header:
 * adding/modifying/removing cards, and shows the header on console as it
 * is modified.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "qfits.h"

/* Dump header with horizontal ruler to make it easier to read */
#define HR "-----------------------------------------------------------\n"
static void hdump(qfits_header * qh)
{
    printf(HR);
    qfits_header_dump(qh, stdout);
    printf(HR);
}

int main(void)
{
    /* Declare a FITS header as a pointer */
	qfits_header	*	qh ;

    /* Create default FITS header and display it */
    printf("Creating default header:\n");
    qh = qfits_header_default();
    hdump(qh);

    /*
     * Add some basic keywords and display each time
     * Notice that keywords are automatically inserted where they are
     * supposed to be placed in the header.
     */
    printf("Adding keyword...\n");
    qfits_header_add(qh, "NAXIS", "3", "3-dimensional", NULL);
    hdump(qh);

    printf("Adding keyword...\n");
    qfits_header_add(qh, "NAXIS3", "30", "third dimension", NULL);
    hdump(qh);

    printf("Adding keyword...\n");
    qfits_header_add(qh, "NAXIS2", "20", "second dimension", NULL);
    hdump(qh);

    printf("Adding keyword...\n");
    qfits_header_add(qh, "NAXIS1", "10", "first dimension", NULL);
    hdump(qh);

    printf("Adding several keywords...\n");
    /* Add different data types */
    qfits_header_add(qh, "section.key1", "one", "string value", NULL);
    qfits_header_add(qh, "section.key2", "2", "integer value", NULL);
    qfits_header_add(qh, "section.key3", "3.0", "float value", NULL);
    qfits_header_add(qh, "section.key4", "T", "boolean value", NULL);
    hdump(qh);

    printf("Adding keyword between key1 and key2...\n");
    qfits_header_add_after(qh, "section.key1", "QFITS", "42",
                           "no comment", NULL);
    hdump(qh);

    /*
     * Now perform some searches on the header
     * using the dedicated qfits_header methods.
     */
    printf("Retrieving keyword values as strings...\n");

    printf("Keyword        Value as string\n"
           "\n"
           "NAXIS1         \"%s\"\n"
           "NAXIS2         \"%s\"\n"
           "SECTION.KEY1   \"%s\"\n"
           "SECTION.KEY2   \"%s\"\n"
           "SECTION.KEY3   \"%s\"\n"
           "SECTION.KEY4   \"%s\"\n",
           qfits_header_getstr(qh, "naxis1"),
           qfits_header_getstr(qh, "naxis2"),
           qfits_header_getstr(qh, "section.key1"),
           qfits_header_getstr(qh, "section.key2"),
           qfits_header_getstr(qh, "section.key3"),
           qfits_header_getstr(qh, "section.key4"));

    printf(HR);

    printf("Retrieving keyword values as typed variables...\n");
    printf("\n");
    printf("SECTION.KEY1 (string) : %s\n"
           "SECTION.KEY2 (int)    : %d\n"
           "SECTION.KEY3 (float)  : %g\n"
           "SECTION.KEY4 (boolean): %d\n",
           qfits_header_getstr(qh, "section.key1"),
           qfits_header_getint(qh, "section.key2", -1),
           qfits_header_getdouble(qh, "section.key3", -1.0),
           qfits_header_getboolean(qh, "section.key4", -1));

    printf(HR);

    /*
     * Modify the header, delete some cards
     */

    printf("Modifying the header...\n");
    qfits_header_mod(qh, "section.key1", "42", "no comment");
    qfits_header_mod(qh, "section.key2", "42", "no comment");
    qfits_header_mod(qh, "section.key3", "42", "no comment");
    qfits_header_mod(qh, "section.key4", "42", "no comment");
    hdump(qh);

    printf("Removing keys...\n");
    qfits_header_del(qh, "section.key1");
    qfits_header_del(qh, "section.key2");
    qfits_header_del(qh, "section.key3");
    qfits_header_del(qh, "section.key4");
    hdump(qh);

    printf("Destroying header...\n");
    qfits_header_destroy(qh);
    return 0 ;
}
